@@ -1,6 +1,6 @@
 /* -*- linux-c -*-
  *
- * automountd.c
+ * automount.c
  *
  *  Automount daemon for Linux autofs
  *
@@ -16,10 +16,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <limits.h>
-#include <dlfcn.h>
 #include <syslog.h>
 #include <sys/ioctl.h>
-#include <sys/mount.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -28,15 +26,11 @@
 
 #include "automount.h"
 
-const char *program;		/* Initialized with argv[0] */
-const char *version = VERSION;	/* Program version */
+const char *program;		        /* Initialized with argv[0] */
+const char *version = VERSION_STRING;	/* Program version */
 
-static pid_t my_pgrp;		/* The "magic" process group */
-static pid_t my_pid;		/* The pid of this process */
-
-static lookup_init_t lookup_init;
-static lookup_mount_t lookup_mount;
-static lookup_done_t lookup_done;
+static pid_t my_pgrp;		        /* The "magic" process group */
+static pid_t my_pid;		        /* The pid of this process */
 
 static int mounted = 0;
 
@@ -52,7 +46,7 @@ static struct autofs_point {
 	int ioctlfd;		/* File descriptor for ioctls */
 	dev_t dev;		/* "Device" number assigned by kernel */
 	struct pending_mount *mounts; /* Pending mount queue */
-	void *lookup_context;	/* lookup library context pointer */
+	struct lookup_mod *lookup; /* Lookup module */
 } ap;
 
 static int umount_all(void)
@@ -75,13 +69,15 @@ static int umount_all(void)
 
 	while ( (de = readdir(dp)) != NULL ) {
 		if ( strcmp(de->d_name,".") && strcmp(de->d_name,"..") ) {
-			sprintf(path_buf, "%s//%s", ap.path, de->d_name);
+			sprintf(path_buf, "%s/%s", ap.path, de->d_name);
 			if ( !lstat(path_buf,&st) ) {
 				if ( S_ISDIR(st.st_mode) ) {
 					if ( st.st_dev != ap.dev ) {
 						rv = spawnl(_PATH_UMOUNT, _PATH_UMOUNT, path_buf, NULL);
-						if ( rv ) 
+						if ( rv ) {
+							syslog(LOG_NOTICE, "could not unmount %s", path_buf);
 							left++;
+						}
 						else
 							rmdir(path_buf);
 					} else
@@ -152,7 +148,7 @@ static int mount_autofs(char *path)
 	
 	sprintf(options, "fd=%d,pgrp=%u,minproto=%d,maxproto=%d", pipefd[1],
 		(unsigned)my_pgrp, AUTOFS_PROTO_VERSION, AUTOFS_PROTO_VERSION);
-	sprintf(our_name, "automountd(pid%u)", (unsigned)my_pid);
+	sprintf(our_name, "automount(pid%u)", (unsigned)my_pid);
 
 	if ( spawnl(_PATH_MOUNT, _PATH_MOUNT, "-t", "autofs", "-o",
 		    options, our_name, path, NULL) ) {
@@ -276,7 +272,7 @@ static int handle_packet(void)
 
 			/* Wait for go-ahead from main process */
 			kill(getpid(), SIGSTOP);
-			i = lookup_mount(ap.path,pkt.name,pkt.len,ap.lookup_context);
+			i = ap.lookup->lookup_mount(ap.path,pkt.name,pkt.len,ap.lookup->context);
 			_exit(i ? 1 : 0);
 		} else {
 			int status;
@@ -364,11 +360,9 @@ static void initialize(void)
 
 int main(int argc, char *argv[])
 {
-	char *path, *map, *mapfmt, **maparg;
-	void *lookup_lib;
+	char *path, *map, *mapfmt, **mapargv;
 	struct sigaction sa;
-	char map_buf[PATH_MAX];
-	int *lookup_version;
+	int mapargc;
 
 	program = argv[0];
 
@@ -382,39 +376,20 @@ int main(int argc, char *argv[])
 
 	path =    argv[1];
 	map  =    argv[2];
-	maparg = &argv[3];
+	mapargv = &argv[3];
+        mapargc = argc-3;         
 
-	syslog(LOG_INFO, "starting automounter, path = %s, maptype = %s, mapname = %s\n", path, map, (argc < 4) ? "none" : maparg[0]);
+	syslog(LOG_INFO, "starting automounter version %s, path = %s, maptype = %s, mapname = %s", version, path, map, (argc < 4) ? "none" : mapargv[0]);
 
 	if ( (mapfmt = strchr(map,',')) )
 		*(mapfmt++) = '\0';
 
-	sprintf(map_buf, "%s//lookup_%s.so", MYLIBDIR, map);
-
-	if ( !(lookup_lib = dlopen(map_buf, RTLD_NOW|RTLD_GLOBAL)) ) {
-		syslog(LOG_CRIT, "cannot load resolver module \"%s\"\n", map);
+	if ( !(ap.lookup = open_lookup(map, "", mapfmt, argc-3, mapargv)) )
 		exit(1);
-	}
-	
-	lookup_version = (int *) dlsym(lookup_lib, "lookup_version");
-	lookup_init = (lookup_init_t) dlsym(lookup_lib, "lookup_init");
-	lookup_mount = (lookup_mount_t) dlsym(lookup_lib, "lookup_mount");
-	lookup_done = (lookup_done_t) dlsym(lookup_lib, "lookup_done");
-
-	if ( !lookup_version || *lookup_version != AUTOFS_LOOKUP_VERSION ||
-		!lookup_init || !lookup_mount || !lookup_done ) {
-		syslog(LOG_CRIT, "resolver library (%s) version mismatch\n",
-		       map);
-		dlclose(lookup_lib);
-		exit(1);
-	}
-
-	if (lookup_init(mapfmt, argc-3, maparg, &ap.lookup_context)) {
-		exit(1);
-	}
 
 	if (mount_autofs(path) < 0) {
-		syslog(LOG_CRIT, "%s: mount failed!\n", path);
+		syslog(LOG_CRIT, "%s: mount failed!", path);
+		close_lookup(ap.lookup);
 		exit(1);
 	}	
 
@@ -455,10 +430,11 @@ int main(int argc, char *argv[])
 		if (shutdown == SIGUSR2 && umount_autofs(0))
 			shutdown = 0;
 	}
-	lookup_done(ap.lookup_context);
-	umount_autofs(1);
 
-	dlclose(lookup_lib);
+	syslog(LOG_INFO, "shutting down, path = %s", path);
+
+	close_lookup(ap.lookup);
+	umount_autofs(1);
 
 	closelog();
 
